@@ -1,4 +1,4 @@
-{-# LANGUAGE MultiParamTypeClasses, FunctionalDependencies, FlexibleInstances, UndecidableInstances #-}
+{-# LANGUAGE CPP, MultiParamTypeClasses, FunctionalDependencies, FlexibleInstances, UndecidableInstances #-}
 -----------------------------------------------------------------------------
 -- |
 -- Module      :  Data.FingerTree
@@ -31,7 +31,11 @@
 -----------------------------------------------------------------------------
 
 module Data.FingerTree (
+#if TESTING
+	FingerTree(..), Digit(..), Node(..), deep, node2, node3,
+#else
 	FingerTree,
+#endif
 	Measured(..),
 	-- * Construction
 	empty, singleton,
@@ -54,7 +58,6 @@ import Prelude hiding (null, reverse)
 import Control.Applicative (Applicative(pure, (<*>)), (<$>))
 import Data.Monoid
 import Data.Foldable (Foldable(foldMap), toList)
-import Data.Traversable (Traversable(traverse))
 
 infixr 5 ><
 infixr 5 <|, :<
@@ -74,13 +77,14 @@ data ViewR s a
 	deriving (Eq, Ord, Show, Read)
 
 instance Functor s => Functor (ViewL s) where
-	fmap f EmptyL           = EmptyL
+	fmap _ EmptyL           = EmptyL
 	fmap f (x :< xs)        = f x :< fmap f xs
 
 instance Functor s => Functor (ViewR s) where
-	fmap f EmptyR           = EmptyR
+	fmap _ EmptyR           = EmptyR
 	fmap f (xs :> x)        = fmap f xs :> f x
 
+-- | 'empty' and '><'.
 instance Measured v a => Monoid (FingerTree v a) where
 	mempty = empty
 	mappend = (><)
@@ -151,11 +155,15 @@ data FingerTree v a
 	= Empty
 	| Single a
 	| Deep !v !(Digit a) (FingerTree v (Node v a)) !(Digit a)
+#if TESTING
+	deriving Show
+#endif
 
 deep ::  (Measured v a) => 
 	 Digit a -> FingerTree v (Node v a) -> Digit a -> FingerTree v a
 deep pr m sf = Deep ((measure pr `mappendVal` m) `mappend` measure sf) pr m sf
 
+-- | /O(1)/. The cached measure of a tree.
 instance (Measured v a) => Measured v (FingerTree v a) where
 	measure Empty           =  mempty
 	measure (Single x)      =  measure x
@@ -167,15 +175,17 @@ instance Foldable (FingerTree v) where
 	foldMap f (Deep _ pr m sf) =
 		foldMap f pr `mappend` foldMap (foldMap f) m `mappend` foldMap f sf
 
-instance (Measured v a, Eq a) => Eq (FingerTree v a) where
+instance Eq a => Eq (FingerTree v a) where
 	xs == ys = toList xs == toList ys
 
-instance (Measured v a, Ord a) => Ord (FingerTree v a) where
+instance Ord a => Ord (FingerTree v a) where
 	compare xs ys = compare (toList xs) (toList ys)
 
-instance (Measured v a, Show a) => Show (FingerTree v a) where
+#if !TESTING
+instance Show a => Show (FingerTree v a) where
 	showsPrec p xs = showParen (p > 10) $
 		showString "fromList " . shows (toList xs)
+#endif
 
 -- | Like 'fmap', but with a more constrained type.
 fmap' :: (Measured v1 a1, Measured v2 a2) =>
@@ -350,6 +360,7 @@ consDigit :: a -> Digit a -> Digit a
 consDigit a (One b) = Two a b
 consDigit a (Two b c) = Three a b c
 consDigit a (Three b c d) = Four a b c d
+consDigit _ (Four _ _ _ _) = illegal_argument "consDigit"
 
 -- | /O(1)/. Add an element to the right end of a sequence.
 -- Mnemonic: a triangle with the single element at the pointy end.
@@ -365,6 +376,7 @@ snocDigit :: Digit a -> a -> Digit a
 snocDigit (One a) b = Two a b
 snocDigit (Two a b) c = Three a b c
 snocDigit (Three a b c) d = Four a b c d
+snocDigit (Four _ _ _ _) _ = illegal_argument "snocDigit"
 
 -- | /O(1)/. Is this the empty sequence?
 null :: (Measured v a) => FingerTree v a -> Bool
@@ -390,6 +402,7 @@ lheadDigit (Three a _ _) = a
 lheadDigit (Four a _ _ _) = a
 
 ltailDigit :: Digit a -> Digit a
+ltailDigit (One _) = illegal_argument "ltailDigit"
 ltailDigit (Two _ b) = One b
 ltailDigit (Three _ b c) = Two b c
 ltailDigit (Four _ b c d) = Three b c d
@@ -413,6 +426,7 @@ rheadDigit (Three _ _ c) = c
 rheadDigit (Four _ _ _ d) = d
 
 rtailDigit :: Digit a -> Digit a
+rtailDigit (One _) = illegal_argument "rtailDigit"
 rtailDigit (Two a _) = One a
 rtailDigit (Three a b _) = Two a b
 rtailDigit (Four a b c _) = Three a b c
@@ -672,7 +686,7 @@ addDigits4 m1 (Four a b c d) e f g h (Four i j k l) m2 =
 -- point, i.e. that the predicate is /monotonic/.
 split ::  (Measured v a) => 
           (v -> Bool) -> FingerTree v a -> (FingerTree v a, FingerTree v a)
-split _p Empty  =  (Empty, Empty)
+split _ Empty  =  (Empty, Empty)
 split p xs
   | p (measure xs) =  (l, x <| r)
   | otherwise	=  (xs, Empty)
@@ -698,7 +712,8 @@ data Split t a = Split t a t
 
 splitTree ::	(Measured v a) => 
 		(v -> Bool) -> v -> FingerTree v a -> Split (FingerTree v a) a
-splitTree _p _i (Single x) = Split Empty x Empty
+splitTree _ _ Empty = illegal_argument "splitTree"
+splitTree _ _ (Single x) = Split Empty x Empty
 splitTree p i (Deep _ pr m sf)
   | p vpr	=  let	Split l x r	=  splitDigit p i pr
 		   in	Split (maybe Empty digitToTree l) x (deepL r m sf)
@@ -740,7 +755,7 @@ splitNode p i (Node3 _ a b c)
 
 splitDigit :: (Measured v a) => (v -> Bool) -> v -> Digit a ->
 		Split (Maybe (Digit a)) a
-splitDigit p i (One a) = i `seq` Split Nothing a Nothing
+splitDigit _ i (One a) = i `seq` Split Nothing a Nothing
 splitDigit p i (Two a b)
   | p va	= Split Nothing a (Just (One b))
   | otherwise	= Split (Just (One a)) b Nothing
@@ -784,6 +799,10 @@ reverseDigit f (Two a b) = Two (f b) (f a)
 reverseDigit f (Three a b c) = Three (f c) (f b) (f a)
 reverseDigit f (Four a b c d) = Four (f d) (f c) (f b) (f a)
 
+illegal_argument :: String -> a
+illegal_argument name =
+	error $ "Logic error: " ++ name ++ " called with illegal argument"
+
 {- $example
 
 Particular abstract data types may be implemented by defining
